import { h, Component } from 'preact';
export default class extends Component {
    constructor(props) {
        super(props);
        this.isLastParent = (node) => {
            return !node || !node.children || !node.children[0].children;
        };
        this.defaultChange = (node, paths) => {
            return node.name;
        };
        this.init = (paths = []) => {
            const { isLastParent } = this;
            const { tree = [] } = this.props;
            let node = { name: '__root__', children: tree };
            let index = 0;
            for (; index < paths.length; index++) {
                node = node.children[paths[index]];
            }
            while (!isLastParent(node)) {
                const n = paths[index] | 0;
                node = node.children[n];
                paths.push(n);
                index++;
            }
            this.setState({
                paths
            });
        };
        this.changeParent = (target, deepth) => {
            let { paths = [] } = this.state;
            this.init(paths.slice(0, deepth).concat(target.value | 0));
        };
        this.toggleActive = () => {
            const { active } = this.state;
            this.setState({
                active: !active
            });
        };
        this.onPick = (node) => {
            const { onChange = this.defaultChange } = this.props;
            const { paths } = this.state;
            const name = onChange(node, paths);
            this.setState({
                name,
                active: false
            });
        };
        this.changeKeywords = (e) => {
            this.setState({
                keywords: e.target.value
            });
        };
        if (props.isLastParent) {
            this.isLastParent = props.isLastParent;
        }
        this.state = {
            active: false
        };
        this.init();
    }
    componentWillReceiveProps(nextProps) {
        this.props = nextProps;
        this.init();
    }
    render() {
        const { tree = [], onChange, isLastParent, ...props } = this.props;
        const { paths, name, active, keywords = '' } = this.state;
        const { changeParent, onPick, toggleActive, changeKeywords } = this;
        let children = tree;
        return h("span", null,
            h("input", Object.assign({ type: "text" }, props, { value: name, onClick: toggleActive })),
            h("div", { className: `node-selector ${active ? 'is-active' : ''}` },
                h("div", { className: "selector-inner" },
                    h("div", { className: "buttons-panel" },
                        paths.map((index, i) => h("select", { key: `${i}`, className: "inline-block", onChange: (e) => changeParent(e.target, i) }, children.map((item, value) => {
                            const selected = value === index;
                            if (selected) {
                                children = item.children;
                            }
                            return h("option", { value: value, selected: selected }, item.name);
                        }))),
                        h("input", { type: "text", className: "inline-block", placeholder: "quick search", onInput: changeKeywords })),
                    h("div", { className: "item-panel" }, children.map(n => n.name.indexOf(keywords) > -1 && h("a", { href: "javascript:;", onClick: () => onPick(n) }, n.name))),
                    h("a", { href: "javascript:;", className: "button-close", onClick: toggleActive }, "\u00D7"))));
    }
}
//# sourceMappingURL=NodePicker.js.map